#!/bin/bash

if [ $# != 1 ]; then
    echo "Uso: $0 arquivo_a_testar.c"
    exit 1
fi

GOLDEN_NAME="nanicolina_golden"
BIN_NAME="testing_bin"

GOLDEN_SUFFIX="golden"
TESTING_SUFFIX="testing"
ENCODED_SUFFIX="enc"
DECODED_SUFFIX="dec"

chmod a+x $GOLDEN_NAME

##########################################################
echo ----------------------------------------------

gcc $1 -O3 -Wall -o $BIN_NAME

if [[ $? != 0 ]] ; then
    echo ---------------------------------------------
    echo ---------------------------------------------
    echo ERRO Durante a compilação.
    echo ---------------------------------------------
    echo ---------------------------------------------
    exit $rc
fi
echo Compilação - OK
echo ----------------------------------------------


#########################################################
for f in *.in; do

    echo "Testando arquivo $f"

#TESTE 1 - compactar o arquivo com o código submetido e comparar o tamanho do arquivo gerado

    ./$GOLDEN_NAME c $f $f.$GOLDEN_SUFFIX.$ENCODED_SUFFIX &>/dev/null
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo ERRO Durante a compactação com a referência.
        exit $rc
    fi

    ./$BIN_NAME c $f $f.$TESTING_SUFFIX.$ENCODED_SUFFIX &>/dev/null
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo ERRO Durante a compactação com a versão em testes.
        exit $rc
    fi    

    G=`wc -c $f.$GOLDEN_SUFFIX.$ENCODED_SUFFIX  | cut -d' ' -f1`
    T=`wc -c $f.$TESTING_SUFFIX.$ENCODED_SUFFIX | cut -d' ' -f1`
    if [ $G != $T ]
    then
        echo "ERRO Os tamanhos dos arquivos compactados com a versao de referencia e testes deveriam ser iguais."
        exit $rc
    fi

#TESTE 2 - descompactar o arquivo gerado no passo anterior com o descompactador submetido e com a referencia

    ./$GOLDEN_NAME d $f.$TESTING_SUFFIX.$ENCODED_SUFFIX $f.$TESTING_SUFFIX.$ENCODED_SUFFIX.$GOLDEN_SUFFIX.$DECODED_SUFFIX &>/dev/null
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo "ERRO Durante a descompactacao do arquivo compactado com o versao de testes usando a versao de referencia."
        exit $rc
    fi

    ./$BIN_NAME d $f.$TESTING_SUFFIX.$ENCODED_SUFFIX $f.$TESTING_SUFFIX.$ENCODED_SUFFIX.$TESTING_SUFFIX.$DECODED_SUFFIX &>/dev/null
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo "ERRO Durante a descompactação do arquivo compactado com a versao de testes usando a versão em testes."
        exit $rc
    fi
    
    diff $f $f.$TESTING_SUFFIX.$ENCODED_SUFFIX.$GOLDEN_SUFFIX.$DECODED_SUFFIX
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo "ERRO O arquivo compactado com a versao de testes descompactado com a versao de referencia nao e igual ao arquivo original."
        exit $rc
    fi
    diff $f $f.$TESTING_SUFFIX.$ENCODED_SUFFIX.$TESTING_SUFFIX.$DECODED_SUFFIX
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo "ERRO O arquivo compactado com a versao de testes descompactado com a versao de testes nao e igual ao arquivo original."
        exit $rc
    fi
   
#TESTE 3 - descompactar o arquivo de entrada compactado com a referência com o código submetido

    ./$BIN_NAME d $f.$GOLDEN_SUFFIX.$ENCODED_SUFFIX $f.$GOLDEN_SUFFIX.$ENCODED_SUFFIX.$TESTING_SUFFIX.$DECODED_SUFFIX &>/dev/null
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo ERRO Durante a descompactação com a versão em testes.
        exit $rc
    fi
    
    diff $f $f.$GOLDEN_SUFFIX.$ENCODED_SUFFIX.$TESTING_SUFFIX.$DECODED_SUFFIX
    rc=$?
    if [[ $rc != 0 ]] ; then
        echo "ERRO Durante a descompactação usando a versão de testes para descompactar um arquivo compactado com a referência."
        exit $rc
    fi    
   
    echo "Teste $f passou."
done

echo ----------------------------------------------

exit 0
